@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -948,8 +948,9 @@ void __ofono_netreg_set_base_station_name(struct ofono_netreg *netreg,
 		 * but don't emit signal */
 		if (netreg->current_operator == NULL)
 			return;
-	} else
+	} else {
 		netreg->base_station = g_strdup(name);
+	}
 
 	ofono_dbus_signal_property_changed(conn, path,
 						NETWORK_REGISTRATION_INTERFACE,
@@ -1305,8 +1306,11 @@ static void init_registration_status(const struct ofono_error *error,
 	if (netreg->mode == NETWORK_REGISTRATION_MODE_AUTO &&
 		(status == NETWORK_REGISTRATION_STATUS_NOT_REGISTERED ||
 			status == NETWORK_REGISTRATION_STATUS_DENIED ||
-			status == NETWORK_REGISTRATION_STATUS_UNKNOWN))
-		netreg->driver->register_auto(netreg, init_register, netreg);
+			status == NETWORK_REGISTRATION_STATUS_UNKNOWN)) {
+		if (netreg->driver->register_auto)
+			netreg->driver->register_auto(netreg, init_register,
+							netreg);
+	}
 }
 
 void ofono_netreg_strength_notify(struct ofono_netreg *netreg, int strength)
@@ -1603,8 +1607,14 @@ static void netreg_unregister(struct ofono_atom *atom)
 	__ofono_watchlist_free(netreg->status_watches);
 	netreg->status_watches = NULL;
 
-	for (l = netreg->operator_list; l; l = l->next)
+	for (l = netreg->operator_list; l; l = l->next) {
+		struct network_operator_data *opd = l->data;
+
+		if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
+			continue;
+
 		network_operator_dbus_unregister(netreg, l->data);
+	}
 
 	g_slist_free(netreg->operator_list);
 	netreg->operator_list = NULL;
