@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -313,6 +313,9 @@ void ofono_ussd_notify(struct ofono_ussd *ussd, int status, const char *str)
 
 		reply = dbus_message_new_method_return(ussd->pending);
 
+		if (!str)
+			str = "";
+
 		dbus_message_iter_init_append(reply, &iter);
 
 		dbus_message_iter_append_basic(&iter, DBUS_TYPE_STRING,
@@ -427,8 +430,9 @@ static void ussd_cancel_callback(const struct ofono_error *error, void *data)
 		ussd->state = USSD_STATE_IDLE;
 
 		reply = dbus_message_new_method_return(ussd->pending);
-	} else
+	} else {
 		reply = __ofono_error_failed(ussd->pending);
+	}
 
 	__ofono_dbus_pending_reply(&ussd->pending, reply);
 }
